/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    heatFluxFixedValueFvPatchField

Description

SourceFiles
    heatFluxFixedValueFvPatchField.C

\*---------------------------------------------------------------------------*/

#ifndef heatFluxFixedValueFvPatchField_H
#define heatFluxFixedValueFvPatchField_H

#include "fixedValueFvPatchFields.H"

#define CFL 0.5

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                     Class heatFluxFixedValueFvPatch Declaration
\*---------------------------------------------------------------------------*/

template<class Type>
class heatFluxFixedValueFvPatchField
:
    public fixedValueFvPatchField<Type>
{
    // Private data

        //- Temperature field Tin_, on the free solid surface [K]
        Field<Type> Tin_;

        //- Teperature field Tmid_, in the middle of the solid [K]
        Field<Type> Tmid_;


        //- Teperature field Tout_, at the interface between solid and fluid [K]
        Field<Type> Tout_;

        //- k_ field, solid thermal conductivity [W/m/K]
        scalar k_;

        //- rho_ field, solid density [kg/m3]
        scalar rho_;

        //- c_ field, solid specific heat capacity [J/kg/K]
        scalar c_;

        //- dx_ field, half the thickness of the solid wall [m]
        scalar dx_;

        //- Current time index
        label curTimeIndex_;

public:

    //- Runtime type information
    TypeName("heatFluxFixedValue");


    // Constructors

        //- Construct from patch and internal field
        heatFluxFixedValueFvPatchField
        (
            const fvPatch&,
            const Field<Type>&
        );

        //- Construct from patch, internal field and dictionary
        heatFluxFixedValueFvPatchField
        (
            const fvPatch&,
            const Field<Type>&,
            const dictionary&
        );

        //- Construct by mapping given heatFluxFixedValueFvPatchField
        //  onto a new patch
        heatFluxFixedValueFvPatchField
        (
            const heatFluxFixedValueFvPatchField<Type>&,
            const fvPatch&,
            const Field<Type>&,
            const fvPatchFieldMapper&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchField<Type> > clone() const
        {
            return tmp<fvPatchField<Type> >
            (
                new heatFluxFixedValueFvPatchField<Type>(*this)
            );
        }

        //- Construct as copy setting internal field reference
        heatFluxFixedValueFvPatchField
        (
            const heatFluxFixedValueFvPatchField<Type>&,
            const Field<Type>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchField<Type> > clone(const Field<Type>& iF) const
        {
            return tmp<fvPatchField<Type> >
            (
                new heatFluxFixedValueFvPatchField<Type>(*this, iF)
            );
        }


    // Member functions

        // Access

            //- Return the Tin_
            const Field<Type>& Tin() const
            {
                return Tin_;
            }

            //- Return reference to the off value to allow adjustment
            Field<Type>& Tin()
            {
                return Tin_;
            }

            //- Return the Tmid_
            const Field<Type>& Tmid() const
            {
                return Tmid_;
            }

            //- Return reference to the off value to allow adjustment
            Field<Type>& Tmid()
            {
                return Tmid_;
            }

            //- Return the Tout_
            const Field<Type>& Tout() const
            {
                return Tout_;
            }

            //- Return reference to the off value to allow adjustment
            Field<Type>& Tout()
            {
                return Tout_;
            }

            //- Return the k_
            const Field<Type>& k() const
            {
                return k_;
            }

            //- Return reference to the k_ value to allow adjustment
            Field<Type>& k()
            {
                return k_;
            }

            //- Return the rho_
            const Field<Type>& rho() const
            {
                return rho_;
            }

            //- Return reference to the rho_ value to allow adjustment
            Field<Type>& rho()
            {
                return rho_;
            }

            //- Return the c_
            const Field<Type>& c() const
            {
                return c_;
            }

            //- Return reference to the c_ value to allow adjustment
            Field<Type>& c()
            {
                return c_;
            }

            //- Return the dx_
            const Field<Type>& dx() const
            {
                return dx_;
            }

            //- Return reference to the dx_ value to allow adjustment
            Field<Type>& dx()
            {
                return dx_;
            }

        // Mapping functions

            //- Map (and resize as needed) from self given a mapping object
            virtual void autoMap
            (
                const fvPatchFieldMapper&
            );

            //- Reverse map the given fvPatchField onto this fvPatchField
            virtual void rmap
            (
                const fvPatchField<Type>&,
                const labelList&
            );


        // Evaluation functions

            //- Update the coefficients associated with the patch field
            virtual void updateCoeffs();


        //- Write
        virtual void write(Ostream&) const;
        
		  //Integrate 1D heat equation
        void heatEq1D
		  (
		       const scalar& cfl
		  );
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
#   include "heatFluxFixedValueFvPatchField.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
